import obspython as obs
import threading
import time
from pythonosc import dispatcher, osc_server

# --------------------------------------
# Script Settings (IP & Port from UI)
# --------------------------------------
ip = "127.0.0.1"
port = 3032

text_sources = {
    "EOSCurrentCue": "-",
    "EOSCurrentLabel": "-",
    "EOSCurrentTime": "-",
    "EOSCurrentPercent": "-",
    "EOSNextCue": "-",
    "EOSNextCueLabel": "-",
    "EOSNextTime": "-",
    "EOSCmd": "-"
}

update_pending = False
server_thread = None
osc_server_instance = None

# --------------------------------------
# OBS UI Integration
# --------------------------------------
def script_description():
    return "Connects to ETC EOS V3.2> via OSC and updates OBS text sources.\n\nUse the following Text (GDI+) sources in OBS. After the dashes (-) shows examples of what is sent from EOS:\n\nEOSCurrentCue - Q2.1\nEOSCurrentLabel - Chris X DSL Focus\nEOSCurrentTime - 4.0s\nEOSCurrentPercent - 100%\nEOSNextCue - Q3\nEOSNextCueLabel - Chris X DSR Follow\nEOSNextTime - 4.0s\nEOSCmd - LIVE: Cue 2.1:\n\nCreated by Chris @ CPQS . UK 😎"

def script_properties():
    props = obs.obs_properties_create()
    obs.obs_properties_add_text(props, "ip", "OSC IP Address", obs.OBS_TEXT_DEFAULT)
    obs.obs_properties_add_text(props, "ip_help", "Usually the Local IP of the machine\nNot usually Localhost 127.0.0.1 unless testing", obs.OBS_TEXT_INFO)
    obs.obs_properties_add_int(props, "port", "OSC Port", 1024, 65535, 1)
    obs.obs_properties_add_text(props, "port_help", "OSC Port EOS is sending on", obs.OBS_TEXT_INFO)
    return props

def script_defaults(settings):
    obs.obs_data_set_default_string(settings, "ip", "127.0.0.1")
    obs.obs_data_set_default_int(settings, "port", 3032)

def script_update(settings):
    global ip, port
    ip = obs.obs_data_get_string(settings, "ip")
    port = obs.obs_data_get_int(settings, "port")
    restart_osc_server()

# --------------------------------------
# OSC Handlers
# --------------------------------------
def cue_text_handler(address, *args):
    global text_sources, update_pending

    full_text = args[0] if args else ""
    print(f"[OBS-EOS] [MATCHED] {address} -> {full_text}")
    parts = full_text.strip().split()

    if address == "/eos/out/active/cue/text":
        if len(parts) >= 4:
            cue = parts[0].split("/")[1]  # Strip the '1/' part, keep the number
            time = parts[-2]
            percent = parts[-1]
            label = " ".join(parts[1:-2])
            text_sources["EOSCurrentCue"] = "Q" + cue  # Prefix "Q" to the cue number
            text_sources["EOSCurrentLabel"] = label
            text_sources["EOSCurrentTime"] = time + "s"
            text_sources["EOSCurrentPercent"] = percent
        else:
            print(f"[OBS-EOS] ⚠️ Unexpected format in active cue: {full_text}")
            text_sources["EOSCurrentCue"] = "-"
            text_sources["EOSCurrentLabel"] = "-"
            text_sources["EOSCurrentTime"] = "-"
            text_sources["EOSCurrentPercent"] = "-"

    elif address == "/eos/out/pending/cue/text":
        if len(parts) >= 3:
            cue = parts[0].split("/")[1]
            time = parts[-1]
            label = " ".join(parts[1:-1])
            text_sources["EOSNextCue"] = "Q" + cue 
            text_sources["EOSNextCueLabel"] = label
            text_sources["EOSNextTime"] = time + "s"
        else:
            print(f"[OBS-EOS] ⚠️ Unexpected format in pending cue: {full_text}")
            text_sources["EOSNextCue"] = "-"
            text_sources["EOSNextCueLabel"] = "-"
            text_sources["EOSNextTime"] = "-"

    update_pending = True

# OSC Handler for /eos/out/cmd
def cmd_text_handler(address, *args):
    global text_sources, update_pending

    if args:
        full_text = args[0]
        print(f"[OBS-EOS] [MATCHED] {address} -> {full_text}")
        text_sources["EOSCmd"] = full_text

    update_pending = True

# --------------------------------------
# OSC Server Setup
# --------------------------------------
def restart_osc_server():
    global server_thread, osc_server_instance

    if osc_server_instance:
        print("[OBS-EOS] 🔄 Restarting OSC server...")
        osc_server_instance.shutdown()
        osc_server_instance = None

    if server_thread and server_thread.is_alive():
        server_thread.join()

    server_thread = threading.Thread(target=start_osc_server)
    server_thread.daemon = True
    server_thread.start()

def start_osc_server():
    global osc_server_instance

    disp = dispatcher.Dispatcher()
    disp.map("/eos/out/active/cue/text", cue_text_handler)
    disp.map("/eos/out/pending/cue/text", cue_text_handler)
    disp.map("/eos/out/cmd", cmd_text_handler)

    try:
        print(f"[OBS-EOS] 🚀 Starting OSC server on {ip}:{port}")
        osc_server_instance = osc_server.ThreadingOSCUDPServer((ip, port), disp)
        osc_server_instance.serve_forever()
    except Exception as e:
        print(f"[OBS-EOS] ❌ Failed to start OSC server: {e}")

# --------------------------------------
# OBS Tick - Update Text Sources
# --------------------------------------
def script_tick(seconds):
    global update_pending
    if update_pending:
        for source_name, text in text_sources.items():
            set_text(source_name, text)
        update_pending = False

def set_text(source_name, text):
    source = obs.obs_get_source_by_name(source_name)
    if source:
        settings = obs.obs_data_create()
        obs.obs_data_set_string(settings, "text", text)
        obs.obs_source_update(source, settings)
        obs.obs_data_release(settings)
        obs.obs_source_release(source)
        print(f"[obs_eos_osc.py] [OBS-EOS] ✅ Updated '{source_name}' with: {text}")
    else:
        print(f"[OBS-EOS] ⚠️ Text source '{source_name}' not found.")

# --------------------------------------
# Cleanup
# --------------------------------------
def script_unload():
    if osc_server_instance:
        osc_server_instance.shutdown()
        print("[OBS-EOS] 🛑 OSC server shut down.")
