import obspython as obs
import mido
import threading
import time

# Global variables
selected_port = ""
tc_text_source = "TCClock"
current_tc = "00:00:00:00"
stop_thread = False
mtc_thread = None
last_quarter_frame = {}
fps = 30
last_update_time = 0
update_interval = 0.05  # 50ms cooldown between updates (faster updates)

# Timer for constant updates
timer_thread = None

def script_description():
    return "Displays live MIDI Timecode (MTC) from a selected MIDI source to a text source called 'TCClock'."

def script_properties():
    props = obs.obs_properties_create()

    # MIDI Port Dropdown
    midi_ports = mido.get_input_names()
    p = obs.obs_properties_add_list(props, "midi_source", "MIDI Timecode Source", obs.OBS_COMBO_TYPE_LIST, obs.OBS_COMBO_FORMAT_STRING)
    for port in midi_ports:
        obs.obs_property_list_add_string(p, port, port)
    return props

def script_update(settings):
    global selected_port, stop_thread, mtc_thread

    selected_port = obs.obs_data_get_string(settings, "midi_source")

    # Restart MIDI thread if necessary
    stop_thread = True
    if mtc_thread:
        mtc_thread.join()
    stop_thread = False
    mtc_thread = threading.Thread(target=mtc_listener)
    mtc_thread.start()

    # Start a timer for continuous updates
    global timer_thread
    if timer_thread is None or not timer_thread.is_alive():
        timer_thread = threading.Thread(target=update_timer)
        timer_thread.start()

def script_unload():
    global stop_thread, mtc_thread
    stop_thread = True
    if mtc_thread:
        mtc_thread.join()

def update_obs_text():
    """Update the text source with the current timecode."""
    settings = obs.obs_data_create()
    obs.obs_data_set_string(settings, "text", current_tc)
    source = obs.obs_get_source_by_name(tc_text_source)
    if source:
        obs.obs_source_update(source, settings)
        obs.obs_source_release(source)
    obs.obs_data_release(settings)

def parse_mtc(msg):
    """Parse the MIDI Timecode and update the timecode if complete."""
    global last_quarter_frame, current_tc

    if msg.type != 'quarter_frame':
        return

    data = msg.bytes()  # Get raw bytes

    if len(data) >= 2:
        frame_type = (data[1] & 0xF0) >> 4  # Masking the high nibble and shifting it
        frame_value = data[1] & 0x0F  # Masking the low nibble

        if 0 <= frame_type <= 7:
            last_quarter_frame[frame_type] = frame_value

        # Process the timecode when all frames are received
        if len(last_quarter_frame) == 8:
            frames = (last_quarter_frame[1] << 4) | last_quarter_frame[0]
            seconds = (last_quarter_frame[3] << 4) | last_quarter_frame[2]
            minutes = (last_quarter_frame[5] << 4) | last_quarter_frame[4]
            hours = ((last_quarter_frame[7] & 0x1) << 4) | last_quarter_frame[6]
            smpte_type = (last_quarter_frame[7] & 0x6) >> 1  # Extract the SMPTE type

            # Lookup FPS based on SMPTE type
            fps_lookup = {0: 24, 1: 25, 2: 29, 3: 30}
            fps = fps_lookup.get(smpte_type, 30)  # Default to 30 fps if not found

            # Format the timecode (HH:MM:SS:FF)
            current_tc = f"{hours:02}:{minutes:02}:{seconds:02}:{frames:02}"

def mtc_listener():
    """Listen for MIDI messages and parse MTC (non-blocking)."""
    try:
        with mido.open_input(selected_port) as inport:
            while not stop_thread:
                msg = inport.poll()  # Non-blocking
                if msg:
                    parse_mtc(msg)
                time.sleep(0.001)  # Prevent CPU spike
    except Exception as e:
        print(f"Error opening MIDI port '{selected_port}': {e}")


def update_timer():
    """Timer to trigger OBS updates at a fixed interval."""
    global last_update_time
    while not stop_thread:
        current_time = time.time()
        if current_time - last_update_time >= update_interval:
            update_obs_text()  # Update OBS text
            last_update_time = current_time  # Update the last update time
        time.sleep(0.01)  # Sleep for a short time to avoid excessive CPU usage
