import obspython as obs
import threading
import time
from pythonosc import dispatcher, osc_server

# --------------------------------------
# Script Settings (IP & Port from UI)
# --------------------------------------
ip = "127.0.0.1"
port = 3032

text_sources = {
    "QLabCurrentResponse": "-"
}

update_pending = False
server_thread = None
osc_server_instance = None

# --------------------------------------
# OBS UI Integration
# --------------------------------------
def script_description():
    return "Connects to QLab via OSC and updates OBS text sources with the received data from /QLab/OBS/Current/.\n\nUse the Text (GDI+) source 'QLabCurrent' in OBS.\n\nCreated by Chris @ CPQS . UK 😎"

def script_properties():
    props = obs.obs_properties_create()
    obs.obs_properties_add_text(props, "ip", "OSC IP Address", obs.OBS_TEXT_DEFAULT)
    obs.obs_properties_add_text(props, "ip_help", "Usually the Local IP of the machine\nNot usually Localhost 127.0.0.1 unless testing", obs.OBS_TEXT_INFO)
    obs.obs_properties_add_int(props, "port", "OSC Port", 1024, 65535, 1)
    obs.obs_properties_add_text(props, "port_help", "OSC Port QLab is sending on - usually 53000", obs.OBS_TEXT_INFO)
    return props

def script_defaults(settings):
    obs.obs_data_set_default_string(settings, "ip", "127.0.0.1")
    obs.obs_data_set_default_int(settings, "port", 3032)

def script_update(settings):
    global ip, port
    ip = obs.obs_data_get_string(settings, "ip")
    port = obs.obs_data_get_int(settings, "port")
    restart_osc_server()

# --------------------------------------
# OSC Handler
# --------------------------------------
def cue_text_handler(address, *args):
    global text_sources, update_pending

    if args:
        full_text = args[0]
        
        # Detailed logging for every received OSC message
        print(f"[OBS-QLab] [MATCHED] {address} -> {full_text}")
        
        # Log full address and arguments to track incoming messages
        print(f"[OBS-QLab] [LOG] Received OSC Command: {address} with arguments: {args}")

        # Only respond to the specific address "/QLab/OBS/Current/"
        if address == "/QLab/OBS/Current/":
            text_sources["QLabCurrent"] = full_text  # Set the response as the received message
            print(f"[OBS-QLab] [LOG] Updated QLabCurrent with: {full_text}")

    update_pending = True

# --------------------------------------
# OSC Server Setup
# --------------------------------------
def restart_osc_server():
    global server_thread, osc_server_instance

    if osc_server_instance:
        print("[OBS-QLab] 🔄 Restarting OSC server...")
        osc_server_instance.shutdown()
        osc_server_instance = None

    if server_thread and server_thread.is_alive():
        server_thread.join()

    server_thread = threading.Thread(target=start_osc_server)
    server_thread.daemon = True
    server_thread.start()

def start_osc_server():
    global osc_server_instance

    disp = dispatcher.Dispatcher()
    disp.map("/QLab/OBS/Current/", cue_text_handler)  # Listen to the specific address

    try:
        print(f"[OBS-QLab] 🚀 Starting OSC server on {ip}:{port}")
        osc_server_instance = osc_server.ThreadingOSCUDPServer((ip, port), disp)
        osc_server_instance.serve_forever()
    except Exception as e:
        print(f"[OBS-QLab] ❌ Failed to start OSC server: {e}")

# --------------------------------------
# OBS Tick - Update Text Sources
# --------------------------------------
def script_tick(seconds):
    global update_pending
    if update_pending:
        for source_name, text in text_sources.items():
            set_text(source_name, text)
        update_pending = False

def set_text(source_name, text):
    source = obs.obs_get_source_by_name(source_name)
    if source:
        settings = obs.obs_data_create()
        obs.obs_data_set_string(settings, "text", text)
        obs.obs_source_update(source, settings)
        obs.obs_data_release(settings)
        obs.obs_source_release(source)
        print(f"[OBS-QLab] ✅ Updated '{source_name}' with: {text}")
    else:
        print(f"[OBS-QLab] ⚠️ Text source '{source_name}' not found.")

# --------------------------------------
# Cleanup
# --------------------------------------
def script_unload():
    if osc_server_instance:
        osc_server_instance.shutdown()
        print("[OBS-QLab] 🛑 OSC server shut down.")
